# coding: utf-8
"""
PixelArtEditor ver0.5
"""
import tkinter as tk
from tkinter import filedialog
from tkinter import colorchooser
import json
from PIL import Image, ImageDraw

class PxArtEditor(tk.Frame):
	def __init__(self, master:tk.Tk):
		super().__init__(master)
		#  定数 --------------
		self.zoom_ratio = 1.0 # 表示倍率
		self.square_length10 = 10 # 網目の正方形の初期値
		self.square_length = self.square_length10* self.zoom_ratio # 網目の正方形の一辺の長さ（ピクセル）
		self.cvs_art_length = 700 #ピクセル絵を描くキャンバスのサイズ
		self.art_lengt_values = [16, 32, 64, 128, 256, 512, 1024] # 設定セル数
		self.art_width = self.art_height = self.art_lengt_values[0] # ドット絵の幅と高さ
		self.art_data = [[0] * self.art_width for _ in range(self.art_height)]
		# --------------------
		master.title(f'Pixel Art Editor ({self.art_width}x{self.art_height})')
		self.pack()
		# ------- menu
		menubar = tk.Menu(master=self.master)
		filemenu = tk.Menu(menubar, tearoff=0)
		filemenu.add_command(label='新規キャンバス', command=self.new_canvas_menu)
		filemenu.add_command(label='キャンバスクリア', command=lambda: self.art_clear(cvs=self.cvs_art))
		filemenu.add_separator()
		filemenu.add_command(label='データファイル保存', command=self.art_save)
		filemenu.add_command(label='データファイル読み込み', command=self.art_load)
		filemenu.add_separator()
		exportmenu = tk.Menu(filemenu, tearoff=False)
		exportmenu.add_command(label='BMP 形式',
							   command=lambda: self.export_image_file('bmp', self.cvs1_data, self.bg_color, 16, 16))
		exportmenu.add_command(label='PNG 形式',
							   command=lambda: self.export_image_file('png', self.cvs1_data, self.bg_color, 16, 16))
		exportmenu.add_command(label='JPG 形式',
							   command=lambda: self.export_image_file('jpg', self.cvs1_data, self.bg_color, 16, 16))
		filemenu.add_cascade(label='ドット絵のエクスポート（BMP,PNG,JPG）', menu=exportmenu)
		filemenu.add_separator()
		filemenu.add_command(label=u'ドット絵の表示(OS標準表示ソフトで）', command=self.show_image)
		filemenu.add_separator()
		filemenu.add_command(label='終了', command=self.master.quit)
		menubar.add_cascade(label='ファイル', menu=filemenu)
		zoommenu = tk.Menu(menubar, tearoff=0)
		zoommenu.add_command(label='拡大(x 1.1)', command=lambda :self.zoom_display(1.1))
		zoommenu.add_command(label='リセット（等倍）', command=lambda :self.zoom_display(1))
		zoommenu.add_command(label='縮小(x 0.9)', command=lambda :self.zoom_display(0.9))
		menubar.add_cascade(label='表示', menu=zoommenu)
		self.master.config(menu=menubar)
		# -----------frm0
		self.frm0 = tk.Frame(self)
		self.frm0.pack(side=tk.LEFT,anchor=tk.NW)
		self.label_color_title = tk.Label(self.frm0, text='色')
		self.label_color_title.grid(column=0, row=0)
		self.label_pixel_color = tk.Label(self.frm0,
										  text=u'ピクセル')
		self.label_pixel_color.grid(column=0,row=1)
		self.cvs_pixel_color = tk.Canvas(self.frm0,
										 width=20, height=20, bg='white')
		self.cvs_pixel_color.bind('<Button>', self.on_click_pixel_color)
		self.cvs_pixel_color.grid(column=1,row=1)
		self.pixel_color = '#ffffff'
		self.label_bg_color = tk.Label(self.frm0,text=u'背景')
		self.label_bg_color.grid(column=0,row=2)
		self.cvs_bg_color = tk.Canvas(self.frm0,width=25, height=25, bg='white')
		self.cvs_bg_color.bind('<Button>', self.on_click_bg_color)
		self.cvs_bg_color.grid(column=1,row=2)
		self.bg_color = '#ffffff'
		# ------- frm1
		self.frm1 = tk.Frame(self,
							 width=400, height=400)
		self.cvs_art = tk.Canvas(self.frm1,
			width=self.cvs_art_length, height=self.cvs_art_length,
			bg='white',highlightthickness=0,
			scrollregion=(0, 0,
						  self.art_lengt_values[-1]*self.square_length,
						  self.art_lengt_values[-1]*self.square_length))
		xbar = tk.Scrollbar(self.frm1, orient=tk.HORIZONTAL)
		xbar.config(command=self.cvs_art.xview)
		ybar = tk.Scrollbar(self.frm1, orient=tk.VERTICAL)
		ybar.config(command=self.cvs_art.yview)
		self.cvs_art.bind('<Button-1>', self.on_left_click_cvs_art)
		self.cvs_art.bind('<Button-2>', self.on_right_click_cvs_art)
		self.cvs_art.grid(column=0, row=0,padx=10, pady=10)
		xbar.grid(column=0, row=1, sticky=tk.W+tk.E)
		ybar.grid(column=1, row=0, sticky=tk.N+tk.S)
		self.cvs_art.config(xscrollcommand=xbar.set)
		self.cvs_art.config(yscrollcommand=ybar.set)
		self.frm1.pack(side=tk.LEFT)

		self.draw_squares(self.cvs_art)
		self.draw_square_with_selected_color(self.cvs_pixel_color, '#ffffff')
		self.draw_square_with_selected_color(self.cvs_bg_color, '#ffffff')


	def art_clear(self, cvs:tk.Canvas):
		cvs.delete('all')
		for r in range(self.art_height):
			for c in range(self.art_width):
				self.art_data[r][c] = 0
		self.draw_squares(cvs)

	def new_canvas_menu(self):
		self.new_canvas_info = None
		self.new_canvas()
		if self.new_canvas_info is not None:
			self.art_width, self.art_height = self.new_canvas_info
			self.master.title(f'Pixel Art Editor ({self.art_width}x{self.art_height})')
			self.art_data = [[0] * self.art_width for _ in range(self.art_height)]
			self.zoom_ratio = 1.0
			self.square_length = self.square_length10 * self.zoom_ratio
			self.draw_squares(self.cvs_art)

	def new_canvas(self):
		def sub_win_btn_ok():
			self.new_canvas_info = (int(spinbox_x.get()), int(spinbox_y.get()))
			sub_win.destroy()
		def sub_win_btn_cancel():
			self.new_canvas_info = None
			sub_win.destroy()
		sub_win = tk.Toplevel()
		sub_win.title('新規キャンバスのサイズ設定')
		sub_win.geometry("240x180")
		label_sub1 = tk.Label(sub_win, text=u'幅（ピクセル）')
		label_sub1.place(width=100, height=40, x=10, y=10)
		spinbox_x = tk.Spinbox(sub_win, state='readonly',
							   values=self.art_lengt_values)
		spinbox_x.place(width=80, height=40, x=150, y=10)
		label_sub2 = tk.Label(sub_win, text=u'高さ（ピクセル）')
		label_sub2.place(width=100, height=40, x=10, y=50)
		spinbox_y = tk.Spinbox(sub_win, state='readonly',
							   values=self.art_lengt_values)
		spinbox_y.place(width=80, height=40, x=150, y=50)
		button_ok = tk.Button(sub_win, text=u'設定',command=sub_win_btn_ok)
		button_ok.place(width=80, height=40, x=20, y=120)
		button_cancel = tk.Button(sub_win, text=u'キャンセル',command=sub_win_btn_cancel)
		button_cancel.place(width=80, height=40, x=120, y=120)
		self.new_canvas_info = None
		sub_win.grab_set()
		sub_win.focus_set()
		sub_win.transient(self.master)
		self.master.wait_window(sub_win)


	def draw_squares(self, cvs:tk.Canvas):
		for r in range(self.art_height+1):
			cvs.create_line(0, r * self.square_length,
							self.square_length * self.art_width, r * self.square_length
							,fill='black', width=1, tag='squares')
		for c in range(self.art_width+1):
			cvs.create_line(c*self.square_length, 0,
							c*self.square_length, self.square_length*self.art_height
							, fill='black', width=1, tag='squares')
		cvs.update()

	def draw_pixels_on_cvs_data(self, cvs:tk.Canvas, data):
		for r in range(self.art_height):
			for c in range(self.art_width):
				if data[r][c] !=0:
					color = data[r][c]
				else:
					color = self.bg_color
				cvs.create_rectangle(c * self.square_length, r * self.square_length,
									 c * self.square_length + self.square_length, (r+1) * self.square_length
									 , fill=color, outline='black', tag='pix')
		cvs.update()

	def on_left_click_cvs_art(self, e):
		c, r = int(self.cvs_art.canvasx(e.x)/self.square_length), \
			   int(self.cvs_art.canvasy(e.y)/self.square_length)
		if 0<=c<self.art_width and 0<=r<self.art_height:
			self.art_data[r][c] = self.pixel_color
			self.cvs_art.create_rectangle(c * self.square_length, r * self.square_length,
										  (c+1) * self.square_length, (r+1) * self.square_length
									   , fill=self.pixel_color, outline='black', tag='pix')

	def on_right_click_cvs_art(self, e):
		c, r = int(self.cvs_art.canvasx(e.x) / self.square_length), \
			   int(self.cvs_art.canvasy(e.y) / self.square_length)
		if 0<=c<self.art_width and 0<=r<self.art_height:
			self.art_data[r][c] = 0
			self.cvs_art.create_rectangle(c * self.square_length, r * self.square_length,
										  (c+1) * self.square_length, (r+1) * self.square_length
									   , fill=self.bg_color, outline='black', tag='pix')

	def on_click_pixel_color(self, e):
		_, color = colorchooser.askcolor(parent=self, title=u'ピクセルの色')
		if color is not None:
			self.pixel_color = color
			self.draw_square_with_selected_color(self.cvs_pixel_color, color)

	def on_click_bg_color(self, e):
		_, color = colorchooser.askcolor(parent=self, title=u'背景色')
		if color is not None:
			self.bg_color = color
			self.draw_square_with_selected_color(self.cvs_bg_color, color)
			self.draw_pixels_on_cvs_data(self.cvs_art, self.art_data)

	def draw_square_with_selected_color(self, cvs:tk.Canvas, color):
		cvs.create_rectangle(3,3, 20,20, fill=color,outline='black')

	def art_save(self):
		filename = filedialog.asksaveasfilename(
			title='Pixel ART Editor: データの保存',
			filetypes=(('json file', '*.json'),)
		)
		if filename:
			with open(filename, 'w') as f:
				json.dump((self.art_width, self.art_height,self.art_data, self.bg_color), f)

	def art_load(self):
		filename = filedialog.askopenfilename(
			title='Pixel Art Editor: データの読み込み',
			filetypes=(('json file', '*.json'),)
		)
		if filename:
			with open(filename) as f:
				self.art_width, self.art_height,self.art_data, self.bg_color = json.load(f)
			self.draw_pixels_on_cvs_data(self.cvs_art, self.art_data)
			self.draw_square_with_selected_color(self.cvs_bg_color, self.bg_color)

	def export_image_file(self, fmt:str, data:list, bg, width, height):
		filename = filedialog.asksaveasfilename(
			title='Pixel ART Editor: ' + fmt+'形式での'+'イメージファイルの保存',
			filetypes=((fmt+' file', '*.'+fmt),)
		)
		if not filename:
			return
		im = Image.new('RGB',(width, height), '#ffffff')
		draw = ImageDraw.Draw(im)
		for r in range(height):
			for c in range(width):
				if data[r][c] != 0:
					draw.point(((c, r)), fill=data[r][c])
				else:
					draw.point(((c, r)), fill=bg)
		im.save(filename)

	def show_image(self):
		data, bg, width, height = self.art_data, self.bg_color, self.art_width, self.art_height
		im = Image.new('RGB',(width, height), '#ffffff')
		draw = ImageDraw.Draw(im)
		for r in range(height):
			for c in range(width):
				if data[r][c] != 0:
					draw.point(((c, r)), fill=data[r][c])
				else:
					draw.point(((c, r)), fill=self.bg_color)
		im.show()

	def zoom_display(self, ratio):
		if ratio == 1:
			self.zoom_ratio = 1.0
			self.square_length = self.square_length10
		elif ratio > 1:
			if self.zoom_ratio < 2.0:
				self.zoom_ratio *= ratio
				self.square_length = self.square_length10 * self.zoom_ratio
		elif ratio < 1:
			if self.zoom_ratio > 0.5:
				self.zoom_ratio *= ratio
				self.square_length = self.square_length10 * self.zoom_ratio
		else:
			return
		self.cvs_art.delete('all')
		self.draw_squares(self.cvs_art)
		self.draw_pixels_on_cvs_data(self.cvs_art, self.art_data)




def main_loop():
	win = tk.Tk()
	app = PxArtEditor(master=win)
	app.mainloop()

if __name__ == '__main__':
	main_loop()
