# coding: utf-8
import tkinter as tk
from tkinter import filedialog
import json

class Px16ArtEditor(tk.Frame):
	def __init__(self, master:tk.Tk):
		super().__init__(master)
		master.title('pixel art 16X16')
		self.pack()
		self.frm1 = tk.Frame(self)
		# ------- menu
		menubar = tk.Menu(master=self.master)
		filemenu = tk.Menu(menubar, tearoff=0)
		filemenu.add_command(label='キャバスクリア', command=lambda :self.art_clear(cvs=self.cvs1))
		filemenu.add_separator()
		filemenu.add_command(label='データファイル保存', command=self.art_save)
		filemenu.add_command(label='データファイル読み込み', command=self.art_load)
		filemenu.add_separator()
		filemenu.add_command(label='終了', command=self.master.quit)
		menubar.add_cascade(label='px16 ファイル', menu=filemenu)
		self.master.config(menu=menubar)
		# -----------
		self.cvs1 = tk.Canvas(self.frm1, width=400, height=400, bg='white',highlightthickness=0)
		self.cvs1.bind('<Button>', self.on_click)
		self.cvs1.pack(padx=10, pady=10)
		self.frm1.pack()

		self.cvs1_data = [[0]*16 for _ in range(16)]
		self.draw_squares(self.cvs1)

	def art_clear(self, cvs:tk.Canvas):
		cvs.delete('all')
		self.draw_squares(cvs)

	def draw_squares(self, cvs:tk.Canvas):
		for r in range(17):
			cvs.create_line(0, r * 20, 20 * 16, r * 20
							, fill='black', width=1, tag='squares')
		for c in range(17):
			cvs.create_line(c*20, 0, c*20, 20*16
							, fill='black', width=1, tag='squares')
		cvs.update()

	def draw_pixels_on_cvs_data(self, cvs:tk.Canvas, data):
		for r in range(16):
			for c in range(16):
				if data[r][c] == 0:
					color = 'white'
				else:
					color = 'red'
				cvs.create_rectangle(c * 20, r * 20, c * 20 + 20, r * 20 + 20
									 , fill=color, outline='black', tag='pix')
		cvs.update()

	def on_click(self, e):
		c, r = e.x//20, e.y//20
		if 0<=c<=15 and 0<=r<=15:
			self.cvs1_data[r][c] = 1 - self.cvs1_data[r][c]
			self.draw_pixels_on_cvs_data(self.cvs1, self.cvs1_data)

	def art_save(self):
		filename = filedialog.asksaveasfilename(
			title='Pixel ART Editor: データの保存',
			filetypes=(('json file', '*.json'),)
		)
		if filename:
			with open(filename, 'w') as f:
				json.dump(self.cvs1_data, f)

	def art_load(self):
		filename = filedialog.askopenfilename(
			title='Pixel Art Editor: データの読み込み',
			filetypes=(('json file', '*.json'),)
		)
		print(type(filename))
		if filename:
			with open(filename) as f:
				self.cvs1_data = json.load(f)

			self.draw_pixels_on_cvs_data(self.cvs1, self.cvs1_data)

def main_loop():
	win = tk.Tk()
	app = Px16ArtEditor(master=win)
	app.mainloop()

if __name__ == '__main__':
	main_loop()
