# coding: utf-8
import tkinter as tk
from tkinter import filedialog
from tkinter import colorchooser
import json

class Px16ArtEditor(tk.Frame):
	def __init__(self, master:tk.Tk):
		super().__init__(master)
		#  master.resizable(False, False)
		master.title('pixel art 16X16')
		self.pack()
		# ------- menu
		menubar = tk.Menu(master=self.master)
		filemenu = tk.Menu(menubar, tearoff=0)
		filemenu.add_command(label='キャンバスクリア', command=lambda: self.art_clear(cvs=self.cvs1))
		filemenu.add_separator()
		filemenu.add_command(label='データファイル保存', command=self.art_save)
		filemenu.add_command(label='データファイル読み込み', command=self.art_load)
		filemenu.add_separator()
		filemenu.add_command(label='終了', command=self.master.quit)
		menubar.add_cascade(label='px16 ファイル', menu=filemenu)
		self.master.config(menu=menubar)
		# -----------frm0
		self.frm0 = tk.Frame(self)
		self.frm0.pack(side=tk.LEFT, anchor=tk.W)
		self.label_color_title = tk.Label(self.frm0, text='色')
		self.label_color_title.grid(column=0, row=0)
		self.label_pixel_color = tk.Label(self.frm0,
										  text=u'ピクセル')
		self.label_pixel_color.grid(column=0, row=1)
		self.cvs_pixel_color = tk.Canvas(self.frm0,
										 width=20, height=20, bg='white')
		self.cvs_pixel_color.bind('<Button>', self.on_click_pixel_color)
		self.cvs_pixel_color.grid(column=1, row=1)
		self.pixel_color = '#ffffff'
		self.label_bg_color = tk.Label(self.frm0,
									   text=u'背景')
		self.label_bg_color.grid(column=0, row=2)
		self.cvs_bg_color = tk.Canvas(self.frm0,
									  width=25, height=25, bg='white')
		self.cvs_bg_color.bind('<Button>', self.on_click_bg_color)
		self.cvs_bg_color.grid(column=1, row=2)
		self.bg_color = '#ffffff'
		# ------- frm1
		self.frm1 = tk.Frame(self,
							 width=400, height=400)
		self.cvs1 = tk.Canvas(self.frm1, width=322, height=322, bg='white',highlightthickness=0)
		self.cvs1.bind('<Button-1>', self.on_left_click_cvs1)
		self.cvs1.bind('<Button-2>', self.on_right_click_cvs1)
		self.cvs1.pack(side=tk.LEFT,padx=10, pady=10)
		self.frm1.pack()
		# self.frm1.bind('<Button-1>', self.on_lm)


		self.cvs1_data = [[0]*16 for _ in range(16)]
		self.draw_squares(self.cvs1)
		self.draw_squsre_with_selected_color(self.cvs_pixel_color, '#ffffff')
		self.draw_squsre_with_selected_color(self.cvs_bg_color, '#ffffff')


	def art_clear(self, cvs:tk.Canvas):
		cvs.delete('all')
		for r in range(16):
			for c in range(16):
				self.cvs1_data[r][c] = 0
		self.draw_squares(cvs)

	def draw_squares(self, cvs:tk.Canvas):
		for r in range(17):
			cvs.create_line(0, r * 20, 20 * 16, r * 20
							, fill='black', width=1, tag='squares')
		for c in range(17):
			cvs.create_line(c*20, 0, c*20, 20*16
							, fill='black', width=1, tag='squares')
		cvs.update()

	def draw_pixels_on_cvs_data(self, cvs:tk.Canvas, data):
		for r in range(16):
			for c in range(16):
				if data[r][c] !=0:
					color = data[r][c]
				else:
					color = self.bg_color

				cvs.create_rectangle(c * 20, r * 20, c * 20 + 20, r * 20 + 20
									 , fill=color, outline='black', tag='pix')
		cvs.update()

	def on_left_click_cvs1(self, e):
		c, r = e.x//20, e.y//20
		if 0<=c<=15 and 0<=r<=15:
			self.cvs1_data[r][c] = self.pixel_color
			self.cvs1.create_rectangle(c * 20, r * 20, c * 20 + 20, r * 20 + 20
								 , fill=self.pixel_color, outline='black', tag='pix')

	def on_right_click_cvs1(self, e):
		c, r = e.x//20, e.y//20
		if 0<=c<=15 and 0<=r<=15:
			self.cvs1_data[r][c] = 0
			self.cvs1.create_rectangle(c * 20, r * 20, c * 20 + 20, r * 20 + 20
									   , fill=self.bg_color, outline='black', tag='pix')

	def art_save(self):
		filename = filedialog.asksaveasfilename(
			title='Pixel ART Editor: データの保存',
			filetypes=(('json file', '*.json'),)
		)
		if filename:
			with open(filename, 'w') as f:
				json.dump((self.pixel_color, self.bg_color,self.cvs1_data), f)

	def art_load(self):
		filename = filedialog.askopenfilename(
			title='Pixel Art Editor: データの読み込み',
			filetypes=(('json file', '*.json'),)
		)
		if filename:
			with open(filename) as f:
				self.pixel_color, self.bg_color,self.cvs1_data = json.load(f)

			self.draw_squsre_with_selected_color(self.cvs_pixel_color, self.pixel_color)
			self.draw_squsre_with_selected_color(self.cvs_bg_color, self.bg_color)
			self.draw_pixels_on_cvs_data(self.cvs1, self.cvs1_data)

	def on_click_pixel_color(self, e):
		_, color = colorchooser.askcolor()
		if color is not None:
			self.pixel_color = color
			self.draw_squsre_with_selected_color(self.cvs_pixel_color, color)

	def on_click_bg_color(self, e):
		_, color = colorchooser.askcolor()
		if color is not None:
			self.bg_color = color
			self.draw_squsre_with_selected_color(self.cvs_bg_color, color)
			self.cvs1.config(bg=color)

	def draw_squsre_with_selected_color(self, cvs:tk.Canvas, color):
		cvs.create_rectangle(5,5, 20,20, fill=color,outline='black')

def main_loop():
	win = tk.Tk()
	app = Px16ArtEditor(master=win)
	app.mainloop()

if __name__ == '__main__':
	main_loop()
